package com.ekexiu.project.storage.dicom;

import java.util.concurrent.atomic.AtomicInteger;

import org.dcm4che3.data.Attributes;
import org.dcm4che3.data.ElementDictionary;
import org.dcm4che3.data.Tag;
import org.dcm4che3.data.UID;
import org.dcm4che3.data.VR;
import org.dcm4che3.net.ApplicationEntity;
import org.dcm4che3.net.Association;
import org.dcm4che3.net.Connection;
import org.dcm4che3.net.Device;
import org.dcm4che3.net.pdu.AAssociateRQ;
import org.dcm4che3.net.pdu.PresentationContext;


public class DicomBase {
	
    protected static String[] IVR_LE_FIRST = {
            UID.ImplicitVRLittleEndian,
            UID.ExplicitVRLittleEndian,
            UID.ExplicitVRBigEndianRetired
        };

    protected final Device device;
    protected final ApplicationEntity ae;
    protected final Connection conn;
    protected final Connection remote;
    protected final AAssociateRQ rq;
    protected Attributes keys = new Attributes();
    protected Association as;
    protected AtomicInteger totNumMatches = new AtomicInteger();

	protected DicomBase(String title) {
		device = new Device(title);
		ae = new ApplicationEntity(title);
		conn = new Connection();
		remote = new Connection();
		rq = new AAssociateRQ();
        device.addConnection(conn);
        device.addApplicationEntity(ae);
        ae.addConnection(conn);
	}
	
	
	public void setRemote(String title,String hostname,int port){
        rq.setCalledAET(title);
        remote.setHostname(hostname);
        remote.setPort(port);
        conn.setReceivePDULength(Connection.DEF_MAX_PDU_LENGTH);
        conn.setSendPDULength(Connection.DEF_MAX_PDU_LENGTH);
        conn.setMaxOpsInvoked(0);
        conn.setMaxOpsPerformed(0);
        
        conn.setPackPDV(true);
        conn.setConnectTimeout( 0);
        conn.setRequestTimeout(0);
        conn.setAcceptTimeout(0);
        conn.setReleaseTimeout(0);
        conn.setResponseTimeout(0);

        conn.setIdleTimeout(0);
        conn.setSocketCloseDelay(Connection.DEF_SOCKETDELAY);
        conn.setSendBufferSize( 0);
        conn.setReceiveBufferSize( 0);
        conn.setTcpNoDelay(true);
	}
	
	public String getLevel(){
		return "STUDY";
	}
	

	
	public void addAttribute(int tag, String value) {
		VR vr = ElementDictionary.vrOf(tag, keys.getPrivateCreator(tag));
		if (value == null) {

			if (vr == VR.SQ)
				keys.newSequence(tag, 1).add(new Attributes(0));
			else
				keys.setNull(tag, vr);
		} else {
			keys.setString(tag, vr, value);
		}
	}
	
    public void configureServiceClass()  {
    	rq.addPresentationContext(new PresentationContext(1,UID.StudyRootQueryRetrieveInformationModelFIND, IVR_LE_FIRST));
    	keys.setString(Tag.QueryRetrieveLevel, VR.CS, "STUDY");
    	

    	
//    	this.addAttribute(PatientID, null);
//    	
//
//    	
//		 "PatientID",
//         "IssuerOfPatientID",
//         "PatientSex", 
//         "PatientBirthDate",
//         "StudyInstanceUID",
//         "StudyInstanceUID",
//         "SeriesInstanceUID",
//         "StudyID",
//         "AccessionNumber",
//         "StudyDate",
//         "StudyTime",
//         "StudyDescription",
//         "NumberOfStudyRelatedSeries",
//         "NumberOfStudyRelatedInstances"
    }

	
	

}
