package com.ekexiu.project.storage.dicom;

import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

import org.dcm4che3.data.Attributes;
import org.dcm4che3.data.Tag;
import org.dcm4che3.data.UID;
import org.dcm4che3.data.VR;
import org.dcm4che3.net.Association;
import org.dcm4che3.net.DimseRSPHandler;
import org.dcm4che3.net.IncompatibleConnectionException;
import org.dcm4che3.net.Priority;
import org.dcm4che3.net.Status;
import org.dcm4che3.net.pdu.PresentationContext;
import org.jfw.util.json.JsonService;

public class DicomQueryImage extends DicomBase {
	private List<SeriesImage> result = new LinkedList<SeriesImage>();
	private SeriesImage image;
	private StringBuilder sb = new StringBuilder();


	private Attributes.Visitor visitor = new Attributes.Visitor() {

		public boolean visit(Attributes attrs, int tag, VR vr, Object value) throws Exception {
			if (value != null) {
				sb.setLength(0);
				if(vr.prompt(value, attrs.bigEndian(), attrs.getSpecificCharacterSet(vr),
		               100000, sb)){
					if(sb.length()<1){
						return true;
					}
					
					if(tag == Tag.SOPInstanceUID){
						image.setId(sb.toString());
					}else if(tag == Tag.InstanceNumber){
						image.setNum(sb.toString());
					}
				}
			}
			return true;
		}
	};

	protected DicomQueryImage(String title) {
		super(title);
	}

	private void query() throws IOException, InterruptedException {
		DimseRSPHandler rspHandler = new DimseRSPHandler(as.nextMessageID()) {
			@Override
			public void onDimseRSP(Association as, Attributes cmd, Attributes data) {
				super.onDimseRSP(as, cmd, data);
				int status = cmd.getInt(Tag.Status, -1);
				if (Status.isPending(status)) {
					image = new SeriesImage();
					result.add(image);
					try {
						data.accept(visitor, false);
					} catch (Exception e) {
					}
				}
			}
		};
		as.cfind(UID.StudyRootQueryRetrieveInformationModelFIND, Priority.NORMAL, keys, null, rspHandler);
	}

	public List<SeriesImage> query(String title, String hostname, int port, String studyUid,String seriesUid)
			throws IOException, InterruptedException, IncompatibleConnectionException, GeneralSecurityException {
		this.setRemote(title, hostname, port);
		this.rq.addPresentationContext(
				new PresentationContext(1, UID.StudyRootQueryRetrieveInformationModelFIND, IVR_LE_FIRST));
		this.keys.setString(Tag.QueryRetrieveLevel, VR.CS, "IMAGE");
		
		
		this.addAttribute(Tag.StudyInstanceUID, studyUid);
		this.addAttribute(Tag.SeriesInstanceUID, seriesUid);
		this.addAttribute(Tag.SOPInstanceUID,null);
		this.addAttribute(Tag.InstanceNumber,null);
		ExecutorService executorService = Executors.newSingleThreadExecutor();
		ScheduledExecutorService scheduledExecutorService = Executors.newSingleThreadScheduledExecutor();
		this.device.setExecutor(executorService);
		this.device.setScheduledExecutor(scheduledExecutorService);

		try {
			as = ae.connect(conn, remote, rq);
			query();
		} finally {
			if (as != null && as.isReadyForDataTransfer()) {
				as.waitForOutstandingRSP();
				as.release();
			}
			executorService.shutdown();
			scheduledExecutorService.shutdown();
		}
		return this.result;
	}

	public static void main(String args[])
			throws IOException, InterruptedException, IncompatibleConnectionException, GeneralSecurityException {
		DicomQueryImage q = new DicomQueryImage("findscu");
		q.query("DCM4CHEE", "192.168.3.97", 11112, "1.2.840.113663.1500.1.434945515.1.1.20180902.100500.537","1.2.840.113663.1500.1.434945515.2.1.20180902.100500.539");
		
		for(SeriesImage s:q.result){
			System.out.println(JsonService.toJson(s));
		}

	}
}
